/*
 * window.c
 *
 * Copyright 2021 Jonathan Schöbel <jonathan@xn--schbel-yxa.info>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 *
 */

#include <stdlib.h>
#include <gtk/gtk.h>

#include "database.h"

#include "window.h"


SP_Window * SP_Window_new (GtkApplication * app)
{
	static SP_Window * Window;
	Window = malloc (sizeof (SP_Window));

	Window->window = GTK_WINDOW (gtk_application_window_new (app));

	gtk_window_set_title (Window->window, APPLICATION_NAME);
	gtk_window_set_default_size (Window->window, WINDOW_WIDTH, WINDOW_HEIGHT);

	/* Accel Group */
	GtkAccelGroup * accel_group;
	accel_group = gtk_accel_group_new ();
	gtk_window_add_accel_group (Window->window, accel_group);


	/* Widgets */

	GtkWidget * grid;
	GtkWidget * menu_bar;
	GtkWidget * menu_item;
	GtkWidget * menu;
	GtkWidget * menu_item_sub;
	GtkWidget * scrolled_window;
	GtkWidget * action_bar;

	/* Pango */

	PangoAttrList * attr_list;
	PangoAttribute * attr;
	PangoFontDescription * font_description;

	/* Window Box */
	grid = gtk_grid_new ();
	gtk_grid_set_row_homogeneous (GTK_GRID (grid), FALSE);
	gtk_grid_set_column_homogeneous (GTK_GRID (grid), FALSE);
	gtk_grid_set_row_spacing (GTK_GRID (grid), 0);
	gtk_grid_set_column_spacing (GTK_GRID (grid), 0);
	gtk_container_add (GTK_CONTAINER (Window->window), grid);

	/* Menu */
	menu_bar = gtk_menu_bar_new();
	gtk_widget_set_hexpand (menu_bar, TRUE);
	gtk_widget_set_halign (menu_bar, GTK_ALIGN_FILL);
	gtk_grid_attach (GTK_GRID (grid), menu_bar, 0, 0, 3, 1);

	/* Menu -> SmartPurse */
	menu_item = gtk_menu_item_new ();
	gtk_menu_item_set_use_underline (GTK_MENU_ITEM (menu_item), TRUE);
	gtk_menu_item_set_label (GTK_MENU_ITEM (menu_item), "_SmartPurse");
	gtk_container_add (GTK_CONTAINER (menu_bar), menu_item);

	menu = gtk_menu_new ();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM (menu_item), menu);

	/* Menu -> Smartpurse -> Quit */
	menu_item_sub = gtk_image_menu_item_new ();
	gtk_menu_item_set_use_underline (GTK_MENU_ITEM (menu_item_sub), TRUE);
	gtk_menu_item_set_label (GTK_MENU_ITEM (menu_item_sub), "_Quit");
	GTK_IMAGE_MENU_ITEM_SET_ICON (GTK_IMAGE_MENU_ITEM (menu_item_sub), "gtk-quit");

	gtk_widget_add_accelerator (menu_item_sub, "activate", accel_group,
								GDK_KEY_Q, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	g_signal_connect (menu_item_sub, "activate", G_CALLBACK (SP_Window_close), Window);

	gtk_container_add (GTK_CONTAINER (menu), menu_item_sub);
	gtk_widget_show (menu_item_sub);
	/* Menu -> Smartpurse -> Quit */

	gtk_widget_show (menu);

	gtk_widget_show (menu_item);
	/* Menu -> SmartPurse */

	gtk_widget_show (menu_bar);
	/* Menu */

	/* Main */
	/* Back Button */
	Window->back_button = gtk_button_new ();
	gtk_button_set_use_underline (GTK_BUTTON (Window->back_button), TRUE);
	gtk_button_set_label (GTK_BUTTON (Window->back_button), "_Back");
	gtk_button_set_image_position (GTK_BUTTON (Window->back_button), GTK_POS_LEFT);
	GTK_BUTTON_SET_ICON (GTK_BUTTON (Window->back_button), "gtk-go-back");

	g_signal_connect (Window->back_button, "clicked", G_CALLBACK (SP_Window_load_previous_state), Window);

	g_object_set (Window->back_button, "margin", BUTTON_MARGIN, NULL);
	gtk_grid_attach (GTK_GRID (grid), Window->back_button, 0, 1, 1, 1);
	gtk_widget_show (Window->back_button);
	/* Back Button */

	/* Label */
	Window->label = gtk_label_new (NULL);

	attr_list = pango_attr_list_new ();

	font_description = pango_font_description_new ();
	pango_font_description_set_weight (font_description, PANGO_WEIGHT_BOLD);

	attr = pango_attr_font_desc_new (font_description);
	pango_font_description_free (font_description);

	pango_attr_list_insert(attr_list, attr);
	gtk_label_set_attributes (GTK_LABEL (Window->label), attr_list);

	gtk_widget_set_hexpand (Window->label, TRUE);
	gtk_widget_set_halign (Window->label, GTK_ALIGN_CENTER);
	g_object_set (Window->label, "margin-start", LABEL_MARGIN, NULL);
	g_object_set (Window->label, "margin-end", LABEL_MARGIN, NULL);
	gtk_grid_attach (GTK_GRID (grid), Window->label, 1, 1, 1, 1);
	gtk_widget_show (Window->label);
	/* Label */

	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	gtk_grid_attach (GTK_GRID (grid), scrolled_window, 0, 2, 3, 1);

	Window->flow_box = gtk_flow_box_new ();
	gtk_flow_box_set_homogeneous (GTK_FLOW_BOX (Window->flow_box), TRUE);
	gtk_flow_box_set_row_spacing (GTK_FLOW_BOX (Window->flow_box), FLOW_BOX_SPACING);
	gtk_flow_box_set_column_spacing (GTK_FLOW_BOX (Window->flow_box), FLOW_BOX_SPACING);

	gtk_flow_box_set_activate_on_single_click (GTK_FLOW_BOX (Window->flow_box), TRUE);

	gtk_widget_set_hexpand (Window->flow_box, TRUE);
	gtk_widget_set_halign (Window->flow_box, GTK_ALIGN_CENTER);
	gtk_widget_set_vexpand (Window->flow_box, TRUE);
	gtk_widget_set_valign (Window->flow_box, GTK_ALIGN_CENTER);
	g_object_set (Window->flow_box, "margin", FLOW_BOX_MARGIN, NULL);
	gtk_container_add (GTK_CONTAINER (scrolled_window), Window->flow_box);

	gtk_widget_show (Window->flow_box);
	gtk_widget_show (scrolled_window);

	/* Next Button */
	Window->next_button = gtk_button_new ();
	gtk_button_set_use_underline (GTK_BUTTON (Window->next_button), TRUE);
	gtk_button_set_label (GTK_BUTTON (Window->next_button), "_Next");
	gtk_button_set_image_position (GTK_BUTTON (Window->next_button), GTK_POS_RIGHT);
	GTK_BUTTON_SET_ICON (GTK_BUTTON (Window->next_button), "gtk-go-forward");

	g_signal_connect (Window->next_button, "clicked", G_CALLBACK (SP_Window_load_next_state), Window);

	g_object_set (Window->next_button, "margin", BUTTON_MARGIN, NULL);
	gtk_grid_attach (GTK_GRID (grid), Window->next_button, 2, 1, 1, 1);
	gtk_widget_show (Window->next_button);
	/* Next Button */
	/* Main */

	/* Action Bar */
	action_bar = gtk_action_bar_new ();
	gtk_widget_set_hexpand (menu_bar, TRUE);
	gtk_widget_set_halign (menu_bar, GTK_ALIGN_FILL);
	gtk_grid_attach (GTK_GRID (grid), action_bar, 0, 3, 2, 1);
	gtk_widget_show (action_bar);
	/* Action Bar */

	gtk_widget_show (grid);
	/* Window Box */

	/* load initial State */
	Window->count_buttons = 0;
	Window->buttons = malloc (0);
	Window->flow_boxes = malloc (0);

	SP_Window_state_init (Window);
	SP_Window_sync_state (Window);
	/* load initial State */

	return Window;
};

void SP_Window_activate (SP_Window * Window)
{
	gtk_window_present (Window->window);
	return;
};

void SP_Window_state_init (SP_Window * Window)
{
	Window->State = malloc (sizeof (SP_Window_State));
	Window->State->previous = NULL;
	Window->State->next = NULL;
	Window->State->label = "Choose an action";
	Window->State->list = malloc(sizeof (SP_Window_State_Button_List));
	Window->State->list->size = 1;
	Window->State->list->active = 1;
	Window->State->list->buttons = malloc (Window->State->list->size * sizeof (SP_Window_State_Button));
	Window->State->list->buttons[0].label = "Start _Purchase";
	Window->State->list->buttons[0].image_path = NULL;
	Window->State->list->buttons[0].callback = SP_Window_callback_start_purchase;
	Window->State->list->buttons[0].data = NULL;
	Window->State->selection_mode = GTK_SELECTION_NONE;
	Window->State->min_children_per_line = 1;
	Window->State->max_children_per_line = 1;

	return;
};

void SP_Window_callback_state_button (GtkWidget * widget, SP_Window * Window)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
	{
		for (int index = 0; index < Window->count_buttons; index++)
		{
			if (Window->buttons[index] == widget)
			{
				Window->State->list->active = index;
			}

			else
			{
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (Window->buttons[index]), FALSE);
			}
		}
		/* TODO: Workaround as some calls aren't implemented yet. */
		if (Window->State->list->buttons[Window->State->list->active].callback != NULL)
		{
			SP_Window_state_discard (Window);
			if ((*Window->State->list->buttons[Window->State->list->active].callback)(Window, Window->State->list->buttons[Window->State->list->active].data))
			{
				SP_Window_load_next_state (Window->next_button, Window);
			}
		}
	}

	return;
}

bool SP_Window_callback_start_purchase (SP_Window * Window, void * data)
{
	int size;
	SP_Database * Database;
	SP_Database_result_company * companies;

	companies = SP_Database_get_companies (Database, &size);

	SP_Window_state_companies (Window, size, companies);
	return TRUE;
}

void SP_Window_state_companies (SP_Window * Window, int size, SP_Database_result_company * companies)
{
	Window->State->next = malloc (sizeof (SP_Window_State));
	Window->State->next->previous = Window->State;
	Window->State->next->next = NULL;
	Window->State->next->label = "Choise a company";
	Window->State->next->list = malloc(sizeof (SP_Window_State_Button_List));
	Window->State->next->list->size = size;
	Window->State->next->list->active = size;
	Window->State->next->list->buttons = malloc (Window->State->next->list->size * sizeof (SP_Window_State_Button));

	for (int index = 0; index < size; index++)
	{
		Window->State->next->list->buttons[index].label = companies[index].label;
		Window->State->next->list->buttons[index].image_path = g_build_filename (IMAGE_DIRECTORY, LOGO_DIRECTORY, companies[index].image_path, NULL);
		Window->State->next->list->buttons[index].callback = NULL;
		Window->State->next->list->buttons[index].data = (void *) &(companies[index].id);
	}

	Window->State->next->selection_mode = GTK_SELECTION_NONE;
	Window->State->next->min_children_per_line = 1;
	Window->State->next->max_children_per_line = size;

	return;
}

void SP_Window_sync_state (SP_Window * Window)
{
	int index;
	GtkWidget * box;
	GtkWidget * label;
	GtkWidget * image;
	GdkPixbuf * pixbuf;

	for (index = 0; index < Window->count_buttons; index++) {
		gtk_container_remove(GTK_CONTAINER (Window->flow_box), Window->flow_boxes[index]);
	};

	if ( Window->State->previous == NULL )
	{
		gtk_widget_set_sensitive (Window->back_button, FALSE);
	}

	else
	{
		gtk_widget_set_sensitive (Window->back_button, TRUE);
	};

	if ( Window->State->next == NULL )
	{
		gtk_widget_set_sensitive (Window->next_button, FALSE);
	}

	else
	{
		gtk_widget_set_sensitive (Window->next_button, TRUE);
	};

	gtk_label_set_text (GTK_LABEL (Window->label), Window->State->label);

	gtk_flow_box_set_selection_mode (GTK_FLOW_BOX (Window->flow_box), Window->State->selection_mode);
	gtk_flow_box_set_min_children_per_line (GTK_FLOW_BOX (Window->flow_box), Window->State->min_children_per_line);
	gtk_flow_box_set_max_children_per_line (GTK_FLOW_BOX (Window->flow_box), Window->State->max_children_per_line);

	Window->count_buttons = Window->State->list->size;
	Window->buttons = realloc (Window->buttons, Window->count_buttons * sizeof (GtkWidget *) );
	Window->flow_boxes = realloc (Window->flow_boxes, Window->count_buttons * sizeof (GtkWidget *) );

	for (index = 0; index < Window->State->list->size; index++)
	{
		Window->buttons[index] = gtk_toggle_button_new ();

		gtk_widget_set_hexpand (Window->buttons[index], TRUE);
		gtk_widget_set_halign (Window->buttons[index], GTK_ALIGN_FILL);
		gtk_widget_set_vexpand (Window->buttons[index], TRUE);
		gtk_widget_set_valign (Window->buttons[index], GTK_ALIGN_FILL);
		gtk_flow_box_insert (GTK_FLOW_BOX (Window->flow_box), Window->buttons[index], index);

		Window->flow_boxes[index] = GTK_WIDGET (gtk_flow_box_get_child_at_index (GTK_FLOW_BOX (Window->flow_box), index));
		gtk_widget_set_hexpand (Window->flow_boxes[index], TRUE);
		gtk_widget_set_halign (Window->flow_boxes[index], GTK_ALIGN_FILL);
		gtk_widget_set_vexpand (Window->flow_boxes[index], TRUE);
		gtk_widget_set_valign (Window->flow_boxes[index], GTK_ALIGN_FILL);
		g_object_set (Window->buttons[index], "margin", FLOW_BOX_CHILD_MARGIN, NULL);

		box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);

		if (Window->State->list->buttons[index].image_path != NULL)
		{
			pixbuf = gdk_pixbuf_new_from_file_at_scale (Window->State->list->buttons[index].image_path, LOGO_WIDTH, LOGO_HEIGHT, TRUE, NULL);
			image = gtk_image_new_from_pixbuf (pixbuf);
			gtk_box_pack_start (GTK_BOX (box), image, TRUE, TRUE, 0);
			gtk_widget_show (image);
		}

		label = gtk_label_new_with_mnemonic (Window->State->list->buttons[index].label);
		gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);
		gtk_widget_show (label);

		gtk_container_add (GTK_CONTAINER (Window->buttons[index]), box);
		gtk_widget_show (box);

		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (Window->buttons[index]), index == Window->State->list->active);

		g_signal_connect (Window->buttons[index], "clicked", G_CALLBACK (SP_Window_callback_state_button), Window);

		gtk_widget_set_can_default (Window->buttons[index], TRUE);
		gtk_widget_grab_default (Window->buttons[index]);

		gtk_widget_show (Window->buttons[index]);
	};

	return;
};

void SP_Window_load_previous_state (GtkWidget * widget, SP_Window * Window)
{
	Window->State = Window->State->previous;
	SP_Window_sync_state (Window);
	return;
};

void SP_Window_load_next_state (GtkWidget * widget, SP_Window * Window)
{
	Window->State = Window->State->next;
	SP_Window_sync_state (Window);
	return;
};

void SP_Window_state_discard (SP_Window * Window)
{
	SP_Window_State * State;

	for (State = Window->State; State->next != NULL; State = State->next);

	for (; State != Window->State; State = State->previous)
	{
		free (Window->State->next->list->buttons);
		free (Window->State->next->list);
		Window->State->next = NULL;
	}

	return;
}

void SP_Window_close (GtkWidget * widget, SP_Window * Window)
{
	gtk_window_close (Window->window);
	return;
};

void SP_Window_free (SP_Window * Window)
{
	SP_Window_state_discard (Window);
	free (Window->State);
	free (Window);
	return;
};
