/*
 * window.h
 *
 * Copyright 2021 Jonathan Schöbel <jonathan@xn--schbel-yxa.info>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 *
 */

#include <gtk/gtk.h>

#include "config.h"
#include "database.h"


typedef struct SP_Window SP_Window;
typedef struct SP_Window_State_Button
{
	const char * label;
	const char * image_path;
	bool (*callback) (SP_Window *, void *);
	void * data;
} SP_Window_State_Button;

typedef struct SP_Window_State_Button_List
{
	unsigned int size;
	unsigned int active;
	SP_Window_State_Button * buttons;
} SP_Window_State_Button_List;

typedef struct SP_Window_State SP_Window_State;
typedef struct SP_Window_State
{
	SP_Window_State * previous;
	SP_Window_State * next;
	const char * label;
	SP_Window_State_Button_List * list;
	GtkSelectionMode selection_mode;
	unsigned int min_children_per_line;
	unsigned int max_children_per_line;
} SP_Window_State;

typedef struct SP_Window
{
	GtkWindow * window;
	GtkWidget * label;
	GtkWidget * back_button;
	GtkWidget * next_button;
	GtkWidget * flow_box;
	SP_Window_State * State;
	unsigned int count_buttons;
	GtkWidget ** buttons;
	GtkWidget ** flow_boxes;
} SP_Window;

SP_Window * SP_Window_new ();
void SP_Window_activate (SP_Window * Window);
void SP_Window_state_init (SP_Window * Window);
void SP_Window_callback_state_button (GtkWidget * widget, SP_Window * Window);
bool SP_Window_callback_start_purchase (SP_Window * Window, void *);
void SP_Window_state_companies (SP_Window * Window, int size, SP_Database_result_company * companies);
void SP_Window_sync_state (SP_Window * Window);
void SP_Window_load_previous_state (GtkWidget * widget, SP_Window * Window);
void SP_Window_load_next_state (GtkWidget * widget, SP_Window * Window);
void SP_Window_state_discard (SP_Window * Window);
void SP_Window_close (GtkWidget * widget, SP_Window * Window);
void SP_Window_free (SP_Window * Window);

GtkWidget * icon;

#define GTK_IMAGE_MENU_ITEM_SET_ICON(menu_item, icon_name) \
	icon = gtk_image_new_from_icon_name(icon_name, GTK_ICON_SIZE_MENU); \
	gtk_image_menu_item_set_image(menu_item, icon);

#define GTK_BUTTON_SET_ICON(button, icon_name) \
	icon = gtk_image_new_from_icon_name(icon_name, GTK_ICON_SIZE_BUTTON); \
	gtk_button_set_image(button, icon);
